<?php

/**
 * Admin
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la 
 * réglementation relative à la protection des données personnelles (RGPD)
 *
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 * 
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v1.0.0
 * @package     App.Model
 */

App::uses('AppModel', 'Model');

class Admin extends AppModel {

    public $name = 'Admin';
    
    public $validationDomain = 'validation';

    public $validate = [
        'password' => [
            'checkPasswordStrength' => [
                'allowEmpty' => true,
                'required' => false,
                'rule' => ['checkPasswordStrength'],
                'message' => false
            ]
        ],
        'passwd' => [
            [
                'rule' => 'notBlank',
                'message' => 'validation.confirmerPassword'
            ],
            [
                'rule' => [
                    'comparePassword',
                    'password'
                ],
                'message' => 'validation.lesMotsPasseNonIdentiques'
            ]
        ],
        'new_password' => [
            'checkPasswordStrength' => [
                'allowEmpty' => true,
                'required' => false,
                'rule' => ['checkPasswordStrength'],
                'message' => false
            ]
        ],
        'new_passwd' => [
            [
                'rule' => [
                    'comparePassword',
                    'new_password'
                ],
                'message' => 'validation.lesMotsPasseNonIdentiques'
            ]
        ],
        'nom' => [
            [
                'rule' => ['custom', REGEXP_ALPHA_FR],
                'message' => 'validation.seulementLettresAcceptees'
            ]
        ],
        'prenom' => [
            [
                'rule' => ['custom', REGEXP_ALPHA_FR],
                'message' => 'validation.seulementLettresAcceptees'
            ]
        ],
        'email' => [
            [
                'rule' => ['custom', REGEXP_EMAIL_FR],
                'message' => 'validation.adresseMailNonValide'
            ]
        ]
    ];
    
    /**
     * belongsTo associations
     *
     * @var array
     * 
     * @access public
     * @created 18/06/2015
     * @version V0.9.0
     */
    public $belongsTo = [
        'User' => [
            'className' => 'User',
            'foreignKey' => 'user_id'
        ]
    ];

}
